//--------------------------------------------------------------------------
// File and Version Information:
//   Time-stamp: <01/09/11 04:16:07 yuji>
// Description:
//     Exercise for SVX banks and Histogramming
//
//
// Environment:
//	Software developed for the CDF Detector.
//
// Author List:
//
// revision history:
// -----------------
//------------------------------------------------------------------------

#ifndef _svxHist_HH_
#define _svxHist_HH_

//----------------------
// Base Class Headers --
//----------------------
#include <string>
#include "BaBar/Cdf.hh"
class AbsEvent;

#include "Framework/APPModule.hh"
class StorableSiPedInfoSet;

#include "Framework/AbsParmBool.hh"
#include "Framework/AbsParmString.hh"
#include "Framework/AbsParmGeneral.hh"
#include "Framework/AbsParmFilename.hh"

#include "SiliconGeometry/SiDigiCode.hh"
#include "SvxCalibDBTables/SVXDh.Defs.hh"
#include "SvxCalibDBTables/SVXDh.hh"

// for HepTuple
#include <HepTuple/HepFileManager.h>
#include <HepTuple/HepRootFileManager.h>
#include <HepTuple/HepHBookFileManager.h>
#include <HepTuple/HepHBookNtuple.h>
#include <HepTuple/HepNtuple.h>
#include <HepTuple/HepHist1D.h>
#include <HepTuple/Column.h>
#include <HepTuple/Block.h>

class SiVrbInfoSet;

class svxHist : public AppModule {
//--------------------
// Instance Members --
//--------------------

public:
  // Constructors
  svxHist(const char* const theName="svxHist",
	      const char* const theDescription=
                 "Create histogram for Silicon");

  // Destructor
  virtual ~svxHist() {};

  // Operations
  virtual AppResult beginJob(AbsEvent* anEvent);
  virtual AppResult event   (AbsEvent* anEvent);
  virtual AppResult beginRun(AbsEvent* anEvent);
  virtual AppResult endRun  (AbsEvent* anEvent);
  virtual AppResult endJob  (AbsEvent* anEvent);
  virtual AppResult abortJob(AbsEvent* anEvent);
  virtual AppModule* clone(const char* cloneName);

protected:

private:
  typedef AbsParmGeneral<int> AbsParmInt;

  ////////////////////////////////////
  // Helper functions

  // Talk-to parameters
  AbsParmInt _anaBarrelStart;
  AbsParmInt _anaBarrelEnd;
  AbsParmInt _anaHalfLadderStart;
  AbsParmInt _anaHalfLadderEnd;
  AbsParmInt _anaPhiWedgeStart;
  AbsParmInt _anaPhiWedgeEnd;
  AbsParmInt _anaLayerStart;
  AbsParmInt _anaLayerEnd;
  AbsParmInt _anaLadderStart; 
  AbsParmInt _anaLadderEnd;
  AbsParmInt _anaSideStart;
  AbsParmInt _anaSideEnd;

  ////////////////////////////////////
  // Data

  int _eventN;
  bool _runBegan;
  bool _runEnded;
  bool _eventSeen;

  SiVrbInfoSet* _svxDaqInfo;
  SiVrbInfoSet* _islDaqInfo;

  // class variables for Histogram
  HepFileManager* _manager;
  HepHist1D* _hist1D;
  HepHist1D* _adc;
  HepNtuple* _ntuple;

  struct {
    int   b;     // barrel
    int   hl;    // half ladder
    int   pw;    // phi wedge
    int   l;     // layer
    int   s;     // side
    int   strip;
    int   adc;
  } _sihit;
};

#endif

