//--------------------------------------------------------------------------
// File and Version Information:
//
// Description:
//	Class DispBank. This is a simple example of a user module. It
//      prints out each Banks
//
// Environment:
//
// Author List:
//        Yuji Takeuchi
//
// Copyright Information:
//
//------------------------------------------------------------------------

//------------------------
// String Header First  --
//------------------------
#include <string>

//-----------------------
// This Class's Header --
//-----------------------
#include "Exercise/DispBanks.hh"
//-------------
// C Headers --
//-------------
#include <assert.h>
#include <time.h>
//---------------
// C++ Headers --
//---------------
#include <iostream.h>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#ifdef USE_CDFEDM2
#include "Edm/ConstHandle.hh"
#include "StorableBanks/LRID_StorableBank.hh"
#else
#include "Banks/LRID_Bank.hh"
#include "Trybos/TRY_Record_Iter_Same.hh"
#endif
#include "AbsEnv/AbsEnv.hh"
//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

static const char rcsid[] = "$Id: UserExample.cc,v 1.11 1999/10/29 21:58:38 sexton Exp $";

//----------------
// Constructors --
//----------------

DispBanks::DispBanks(
    const char* const theName, 
    const char* const theDescription )
    : AppModule( theName, theDescription )
{
}


//--------------
// Destructor --
//--------------

DispBanks::~DispBanks( )
{
}

//--------------
// Operations --
//--------------

AppResult DispBanks::beginJob( AbsEvent* aJob )
{
  _eventN = 0;  // clear the event number
  
  cout << name( ) << " begin Job" << endl;
  return AppResult::OK;
}

AppResult DispBanks::beginRun( AbsEvent* aRun )
{
  cout << name( ) << " begin Run" << endl;
  return AppResult::OK;
}

AppResult DispBanks::event( AbsEvent* anEvent )
{
  cout << name( ) << " event entry" << endl;

  ++_eventN; // keep track of how many events we've processed
  cout << name() << "::event: physics event " << _eventN << endl;

// For CDF users, this is an example of how to access event data in a user
// written module.  Note that a pointer to the event is passed to the event
// entry, and that entry returns the (optionally) modified event.
//

  // listClass
  //   if listClass parameter is set, list up all banks in the eventRecord.
  for (EventRecord::ConstIterator it(anEvent);
       it.is_valid(); it++ ) {
    cout <<"class[" << it->object_id() << "]"
	 << ": " << it->class_name() << endl;
  }
  return AppResult::OK;
}

AppResult DispBanks::other( AbsEvent* )
{
    cout << name( ) << " other" << endl;
    return AppResult::OK;
}

AppResult DispBanks::endRun( AbsEvent* aRun )
{
    cout << name( ) << " end Run" << endl;
    return AppResult::OK;
}

AppResult DispBanks::endJob( AbsEvent* aJob )
{
    cout << name( ) << " end Job" << endl;
    return AppResult::OK;
}

AppResult DispBanks::abortJob( AbsEvent* aJob )
{
    cout << name( ) << " abort Job" << endl;
    return AppResult::OK;
}

AppModule*
DispBanks::clone(const char* cloneName)
{
  return new DispBanks(cloneName,"this module is a clone DispBanks");
}

const char * 
DispBanks::rcsId( ) const
{
   return rcsid;
}
