//--------------------------------------------------------------------------
// File and Version Information:
//
// Description:
//    Just for my exercise, this program is example that create histogram
//    related to Banks relevant to Silicon.
//
// Environment:
//
// Author List:
//
// Copyright Information:
//
//------------------------------------------------------------------------

//---------------
// C++ Headers --
//---------------
#include <iostream>
#include <ios>
#include <iomanip>
#include <vector>

//-----------------------
// This Class's Header --
//-----------------------
#include "Exercise/svxHist.hh"

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include <AbsEnv/AbsEnv.hh>

#ifdef USE_CDFEDM2
#include "Edm/ConstHandle.hh"
#include "TrackingObjects/Storable/StorableRun2SiStripSet.hh"
#endif
#include "SiliconGeometry/AbsSiNumerology.hh"
#include "SiliconGeometry/HWSiNumerology.hh"
#include "CalibDB/RunListKey.hh"
#include "SvxDaqUtils/SiDReadVec.hh"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

//		----------------------------------------
// 		-- Public Function Member Definitions --
//		----------------------------------------

//----------------
// Constructors --
//----------------

svxHist::svxHist(const char* const theName, 
                           const char* const theDescription)
  : AppModule(theName, theDescription )
  , _anaBarrelStart("anaBarrelStart",this,0)
  , _anaBarrelEnd("anaBarrelEnd",this, HWSiNumerology::N_BARRELS_MAX-1)
  , _anaHalfLadderStart("anaHalfLadderStart",this,0)
  , _anaHalfLadderEnd("anaHalfLadderEnd",this,2-1)
  , _anaPhiWedgeStart("anaPhiWedgeStart",this,0)
  , _anaPhiWedgeEnd("anaPhiWedgeEnd",this, HWSiNumerology::N_PHI_MAX-1)
  , _anaLayerStart("anaLayerStart",this,0)
  , _anaLayerEnd("anaLayerEnd",this, HWSiNumerology::N_LAYERS_MAX-1)
  , _anaLadderStart("anaLadderStart",this,0)
  , _anaLadderEnd("anaLadderEnd",this,0)
  , _anaSideStart("anaSideStart",this,0)
  , _anaSideEnd("anaSideEnd",this, HWSiNumerology::N_READOUT_SIDES_MAX-1)
  , _runBegan(false)
  , _runEnded(false)
  , _eventSeen(false)
{
#ifdef USE_CDFEDM2
  _anaBarrelStart.addDescription( string(
    "\t\t\tFirst Barrel to Analyze  set <n>"));
  _anaBarrelEnd.addDescription( string(
    "\t\t\tLast Barrel to Analyze  set <n>"));
  _anaHalfLadderStart.addDescription( string(
    "\t\t\tFirst HalfLadder to Analyze  set <n>"));
  _anaHalfLadderEnd.addDescription( string(
    "\t\t\tLast HalfLadder to Analyze  set <n>"));
  _anaPhiWedgeStart.addDescription( string(
    "\t\t\tFirst PhiWedge to Analyze  set <n>"));
  _anaPhiWedgeEnd.addDescription( string(
    "\t\t\tLast PhiWedge to Analyze  set <n>"));
  _anaLayerStart.addDescription( string(
    "\t\t\tFirst Layer to Analyze  set <n>"));
  _anaLayerEnd.addDescription( string(
    "\t\t\tLast Layer to Analyze  set <n>"));
  _anaLadderStart.addDescription( string(
    "\t\t\tFirst Ladder to Analyze  set <n>"));
  _anaLadderEnd.addDescription( string(
    "\t\t\tLast Ladder to Analyze  set <n>"));
  _anaSideStart.addDescription( string(
    "\t\t\tFirst Side to Analyze  set <n>"));
  _anaSideEnd.addDescription( string(
    "\t\t\tLast Side to Analyze  set <n>"));

  commands()->append(&_anaBarrelStart);
  commands()->append(&_anaBarrelEnd);
  commands()->append(&_anaHalfLadderStart);
  commands()->append(&_anaHalfLadderEnd);
  commands()->append(&_anaPhiWedgeStart);
  commands()->append(&_anaPhiWedgeEnd);
  commands()->append(&_anaLayerStart);
  commands()->append(&_anaLayerEnd);
  commands()->append(&_anaLadderEnd);
  commands()->append(&_anaLadderEnd);
  commands()->append(&_anaSideStart);
  commands()->append(&_anaSideEnd);

#endif // USE_CDFEDM2
}

//--------------
// Operations --
//--------------

/*
 *
 *  beginJob
 *
 */
AppResult svxHist::beginJob( AbsEvent* anEvent ) {
  _eventN = 0;
  _runBegan=true;
  _runEnded=false;
  _eventSeen=false;

  // set up the Daq Info Sets
  SiVrbInfoSet::DetType detType;
  _svxDaqInfo = new SiVrbInfoSet();
  detType = SiVrbInfoSet::SVXII;
  _svxDaqInfo->daqInfoSetUp(detType);
  _islDaqInfo = new SiVrbInfoSet();
  detType = SiVrbInfoSet::ISL;
  _islDaqInfo->daqInfoSetUp(detType);
  cout << name() << ": SvxPedModule: set up daqinfo set" << endl;

  //
  // Open root file
  //
  cout << name() << ": Root file open" << endl;
  _manager = new HepRootFileManager("svxHist.root",
				    HepFileManager::HEP_REMOVE);

  // define hist1D
  _hist1D = &_manager->hist1D( "RAW ADC", 128, 0.0 , 128.0 );
  _adc = &_manager->hist1D( "ADC", 256, 0.0 , 256.0 );

  // define ntuple
  char* chform_sihit="b:I*4,hl:I*4,pw:I*4,l:I*4,s:I*4,"
    "strip:I*4,adc:I*4";

  vector<void*> sihit_addr;
  sihit_addr.push_back(&_sihit.b);
  sihit_addr.push_back(&_sihit.hl);
  sihit_addr.push_back(&_sihit.pw);
  sihit_addr.push_back(&_sihit.l);
  sihit_addr.push_back(&_sihit.s);
  sihit_addr.push_back(&_sihit.strip);
  sihit_addr.push_back(&_sihit.adc);

  _ntuple = &_manager->ntuple("NTUPLE",1);
  _ntuple->setColumnWise();
  _ntuple->block("sihit", chform_sihit, sihit_addr);

  // end of begin Job
  cout << name() << ": Begin Job Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  endJob
 *
 */
AppResult svxHist::endJob( AbsEvent* anEvent ) {
  cout << name() << ": End Job Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  beginRun
 *
 */
AppResult svxHist::beginRun( AbsEvent* anEvent ) {
  cout << name() << ": Run# " << gblEnv->runNumber() << endl;
  cout << name() << ": Begin Run Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  abortJob
 *
 */
AppResult svxHist::abortJob( AbsEvent* anEvent ) {
  cout << name() << ": Abort Job Successful" << endl;
  return AppResult::OK;
}

/*
 *
 *  endRun
 *
 */
AppResult svxHist::endRun( AbsEvent* anEvent ) {
  _runBegan=false;
  _runEnded=true;
  _eventSeen=false;
  cout << name() << ": End Job Successful" << endl;
  return AppResult::OK;
}


/*
 *
 *  clone
 *
 */
AppModule* svxHist::clone( const char* cloneName ) {
  return new svxHist( cloneName,
			  "This module is a clone of svxHist." );
}


/*
 *
 *  event
 *
 */
AppResult svxHist::event( AbsEvent* anEvent){
  if(!_runBegan){
    cout << "Event seen but no begin run --> call begin run"<<endl;
    AppResult stat=beginRun(anEvent);
    if(stat!=AppResult::OK){
      return stat;
    }
  }
  _eventSeen=true;
  ++_eventN; // keep track of how many events we've processed

  cout << name() << ": physics event " << _eventN << endl;

  // create StorableRun2SiStripSet
  //   create StorableRun2SiStripSet
  //   and then, entry it to the eventRecord.
  EventRecord::ConstIterator* piter=
    new EventRecord::ConstIterator(anEvent, "StorableRun2SiStripSet");
  if ( !piter->is_valid() ) {
    StorableRun2SiStripSet *ps=0;
    const SiStripSet *myStrips = ps = new StorableRun2SiStripSet(anEvent);
    // Probably these line below may not be necessary.
    //    GeometricRegion & region = TWorld;
    //    ps->loadFromBanks(anEvent, region );

    //    ps->writeBanks( anEvent );

    Handle<StorableRun2SiStripSet> h(ps);
    anEvent->append(h);

    // Now we can get a new iterator.
    delete piter;
    piter= new EventRecord::ConstIterator(anEvent, "StorableRun2SiStripSet");
  }

  // listStrips
  //   if listStrips is set, scan all strip if StorableRun2SiStripSet exist,
  //   and list all strip exist in the eventRecord.
  // 28-Sep-2000
  //   attempt new iteration frame
  if( piter->is_valid() ) {
    ConstHandle<StorableRun2SiStripSet> myStrips(*piter);
    cout << "Now iterating over all strips" << endl;

    // iterate over all the strips.
    // Loop over barrel, halfLadder, phiWedge, layer, ladder and side.
    // -- What is ladder? It seems to be completely dummy.
    for( unsigned phiWedge=_anaPhiWedgeStart.value();
	 phiWedge<=_anaPhiWedgeEnd.value(); phiWedge++ )
      for( unsigned barrel=_anaBarrelStart.value();
	   barrel<=_anaBarrelEnd.value(); barrel++ )
	for( unsigned halfLadder=_anaHalfLadderStart.value();
	     halfLadder<=_anaHalfLadderEnd.value(); halfLadder++ )
	  for( unsigned layer=_anaLayerStart.value();
	       layer<=_anaLayerEnd.value(); layer++ )
	    for( unsigned ladder=_anaLadderStart.value();
		 ladder<=_anaLadderEnd.value(); ladder++ )
	      for( unsigned side=_anaSideStart.value();
		   side<=_anaSideEnd.value(); side++ ) {
		// set SiDigiCode of this ladder.
		SiDigiCode detector( barrel, halfLadder, phiWedge,
				       layer, ladder, side);
		// check validity of this ladder.
		if( myStrips->nStrips(detector) <= 0 ) {
		  // might be invalid, skip this ladder.
		  continue;
		}
		
		// initialization before iteration over strips here.
		unsigned minStrip(9999), maxStrip(0);
		
		// This for-loop is
		// iteration over all strips in this ladder.
		for (SiStripSet::ConstIterator
		       s=myStrips->beginDetector(detector);
		     s!=myStrips->endDetector(detector); ++s) {
		  
		  // from now on, we describe iterated task.
		  _hist1D->accumulate(s->getStripNumber()%128,
				      s->getPulseHeight());
		  _adc->accumulate(s->getPulseHeight());

		  // stuff data into ntuple
		  _sihit.b  = detector.getBarrel();
		  _sihit.hl = detector.getHalfLadder();
		  _sihit.pw = detector.getPhiWedge();
		  _sihit.l  = detector.getLayer();
		  _sihit.s  = detector.getPnSide();
		  _sihit.strip= s->getStripNumber();
		  _sihit.adc= s->getPulseHeight();
		  _ntuple->captureBlock("sihit");
		  _ntuple->storeCapturedData();
		  _ntuple->clearDataBlock("sihit");
		  
		  if( minStrip>s->getStripNumber() )
		    minStrip=s->getStripNumber();
		  if( maxStrip<s->getStripNumber() )
		    maxStrip=s->getStripNumber();

		} // end of loop for strips
		
		  // end task of iteration over strips here.
		cout << name() << ": B:" << detector.getBarrel()
		     << " HL:" << detector.getHalfLadder()
		     << " PW:" << detector.getPhiWedge()
		     << " L:" << detector.getLayer()
		     << " S:" << detector.getPnSide()
		     << " strip:" << minStrip << " - " << maxStrip
		     << " (" << myStrips->nStrips(detector) << ")"
		     <<endl;
		
	      } // end of loop for side, ... and barrel
  } // correspond to if( iter.is_valid() )
  else {
      cout << name() << ": StorableRun2SiStripSet is invalid." << endl;
  }

  return AppResult::OK;
}

/* EOF */
