//--------------------------------------------------------------------------
// File and Version Information:
//
// Description:
//	Class template.
//
// Environment:
//
// Author List:
//
// Copyright Information:
//
//------------------------------------------------------------------------

//------------------------
// String Header First  --
//------------------------
#include <string>

//-----------------------
// This Class's Header --
//-----------------------
#include "Exercise/TEMPLATE.hh"
//-------------
// C Headers --
//-------------
#include <assert.h>
#include <time.h>
//---------------
// C++ Headers --
//---------------
#include <iostream.h>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "Edm/ConstHandle.hh"
#include "StorableBanks/LRID_StorableBank.hh"

#include "AbsEnv/AbsEnv.hh"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

static const char rcsid[] = "$Id: $";

//----------------
// Constructors --
//----------------

TEMPLATE::TEMPLATE( 
		   const char* const theName, 
		   const char* const theDescription ) :
  AppModule( theName, theDescription )
{
}


//--------------
// Destructor --
//--------------

TEMPLATE::~TEMPLATE()
{
}

//--------------
// Operations --
//--------------

AppResult TEMPLATE::beginJob( AbsEvent* aJob )
{
  cout << name( ) << " begin Job" << endl;
  return AppResult::OK;
}

AppResult TEMPLATE::beginRun( AbsEvent* aRun )
{
  cout << name( ) << " begin Run" << endl;
  return AppResult::OK;
}

AppResult TEMPLATE::event( AbsEvent* anEvent )
{
  cout << name( ) << " event entry" << endl;

// For CDF users, this is an example of how to access event data in a user
// written module.  Note that a pointer to the event is passed to the event
// entry, and that entry returns the (optionally) modified event.
//
  EventRecord::ConstIterator iLRID( anEvent, "LRID_StorableBank") ;
  if (iLRID.is_valid()) {
    ConstHandle<LRID_StorableBank> h_LRID(iLRID) ;
    h_LRID->print();
  }

  cout << "Global run, trigNumber, and monteFlags: " <<
    gblEnv->runNumber() <<
    "   " << gblEnv->trigNumber() <<
    "   " << gblEnv->monteFlag() << endl;
    
  return AppResult::OK;
}

AppResult TEMPLATE::other( AbsEvent* )
{
  cout << name( ) << " other" << endl;
  return AppResult::OK;
}

AppResult TEMPLATE::endRun( AbsEvent* aRun )
{
  cout << name( ) << " end Run" << endl;
  return AppResult::OK;
}

AppResult TEMPLATE::endJob( AbsEvent* aJob )
{
  cout << name( ) << " end Job" << endl;
  return AppResult::OK;
}

AppResult TEMPLATE::abortJob( AbsEvent* aJob )
{
  cout << name( ) << " abort Job" << endl;
  return AppResult::OK;
}

AppModule* TEMPLATE::clone(const char* cloneName)
{
  return new TEMPLATE(cloneName,"this module is a clone TEMPLATE");
}

const char * TEMPLATE::rcsId( ) const
{
  return rcsid;
}
